import logging

import boto3

from instance_control import node
from instance_control import piu
from instance_control.command import Command

_LOG = logging.getLogger('bubuku.cluster.command.terminate')


class TerminateCommand(Command):
    def __init__(self, cluster_config_path: str, ip: str, user: str, odd: str):
        super().__init__(cluster_config_path)
        self.ip = ip
        self.user = user
        self.odd = odd

    def execute(self):
        ec2_resource = boto3.resource('ec2', region_name=self.cluster_config['region'])
        instance = node.get_instance_by_ip(ec2_resource, self.cluster_config['cluster_name'], self.ip)
        piu.stop_taupage(self.ip, self.user, self.odd)
        node.terminate(self.cluster_config['cluster_name'], instance)



import logging

import boto3

_LOG = logging.getLogger('bubuku.cluster.aws.security_group')


def create_or_ger_security_group(cluster_config: dict) -> dict:
    _LOG.info('Configuring security group ...')
    ec2 = boto3.client('ec2', cluster_config['region'])
    security_groups = ec2.describe_security_groups(
        Filters=[{'Name': 'group-name', 'Values': [cluster_config['cluster_name']]}])
    if security_groups['SecurityGroups']:
        sg = security_groups['SecurityGroups'][0]
        _LOG.info('Security group for %s exists, will use it %s', cluster_config['cluster_name'], sg['GroupId'])
        return sg

    _LOG.info("Security group does not exists, creating ...")
    sg_name = cluster_config['cluster_name']
    sg = ec2.create_security_group(GroupName=sg_name, VpcId=cluster_config['vpc_id'],
                                   Description='Bubuku Security Group')
    _LOG.info("Security group %s is created", sg_name)
    ec2.create_tags(Resources=[sg['GroupId']], Tags=[{'Key': 'Name', 'Value': sg_name}])
    ec2.authorize_security_group_ingress(GroupId=sg['GroupId'],
                                         IpPermissions=[get_ip_permission(22), get_ip_permission(8004),
                                                        get_ip_permission(8778), get_ip_permission(9100),
                                                        get_ip_permission(9092),
                                                        get_ip_permission(
                                                            cluster_config['environment']['health_port'])])
    _LOG.info("Security group got ingress for ports: 22, 8004, 8080, 8778, 9100, %s",
              cluster_config['environment']['health_port'])
    return sg


def get_ip_permission(port: int):
    return {
        'IpProtocol': 'tcp',
        'FromPort': port,
        'ToPort': port,
        'IpRanges': [{'CidrIp': '0.0.0.0/0'}]
    }



import logging

import boto3

from instance_control.command import Command

_LOG = logging.getLogger('bubuku.cluster.command.get')


class GetCommand(Command):
    def __init__(self, cluster_config_path: str):
        super().__init__(cluster_config_path)

    def execute(self):
        ec2_resource = boto3.resource('ec2', region_name=self.cluster_config['region'])
        instances = list(ec2_resource.instances.filter(Filters=[
            {'Name': 'instance-state-name', 'Values': ['running', 'pending']},
            {'Name': 'tag:Name', 'Values': [self.cluster_config['cluster_name']]}]))

        if len(instances) == 0:
            _LOG.info('Cluster has no running instances')
            return

        max_ip = max([len(instance.private_ip_address) for instance in instances])
        max_id = max([len(instance.instance_id) for instance in instances])

        pattern = '{0:' + str(max_ip) + '}   {1:' + str(max_id) + '}   {2}'
        print(pattern.format('ip', 'id', 'volume'))
        for instance in instances:
            print(pattern.format(instance.private_ip_address,
                                 instance.instance_id,
                                 [x['Ebs']['VolumeId'] for x in instance.block_device_mappings if
                                  x['DeviceName'] == '/dev/xvdk'][0]))
